/*==============================================================================

    S E N S O R S   AMBIENT LIGHT AND PROXIMITY  D R I V E R

DESCRIPTION

   Implements the driver for the ap3426 ALS(Light) and Proximity Sensor

Copyright (c) 2015 by QUALCOMM Technologies, Incorporated.  All Rights Reserved.
Qualcomm Technologies Proprietary and Confidential
==============================================================================*/
/*==============================================================================

                      EDIT HISTORY FOR FILE

  This section contains comments describing changes made to the module.
  Notice that changes are listed in reverse chronological order.

$Header: //source/qcom/qct/core/pkg/dsps/rel/2.2/adsp_proc/Sensors/dd/qcom/src/sns_dd_alsprx_ap3426.c#3 $


***************************************************************************
* REVISON HISTORY
* VERSION   DATE(MM/DD/YY)  WHO     DESCRIPTION
* 3.0       03/10/2017      LO      Refactoring scenario of als auto gain
* 2.0       06/18/2016      LO      Add a compiled flag for enable/disable uImage
* 1.0       12/22/2015      LO      Initial version of driver for AP3426
***************************************************************************
==============================================================================*/
/*============================================================================
                                INCLUDE FILES
============================================================================*/
#include "sns_dd_alsprx_ap3426_priv.h"

#include "sns_memmgr.h"
#include "log_codes.h"
#include "sns_debug_str.h"
#include "sns_debug_api.h"
#include "sns_log_types.h"
#include "sns_log_api.h"


/*============================================================================
                            STATIC VARIABLE DEFINITIONS
============================================================================*/
static const sns_ddf_sensor_e sns_dd_ap3426_alsprx_sensor_types[SNS_DD_AP3426_ALSPRX_NUM_SENSOR_TYPES] =
{
  SNS_DDF_SENSOR_PROXIMITY,
  SNS_DDF_SENSOR_AMBIENT
};

/*===========================================================================
FUNCTION      sns_dd_ap3426_update_registers

DESCRIPTION   update registers
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_update_registers(sns_dd_ap3426_alsprx_state_t* state)
{
    uint8_t byteVal;
    ap3426_uword_u wordVal;

    ap3426_lsc_setup*       setup;
    sns_ddf_status_e status = SNS_DDF_SUCCESS;
    setup  = &state->chip.setup;

    /* Reset the device sns_dd_ap3426_set_default_reg_values */
    DDF_MSG_0(MED, "ap3426_update_registers() start");

    /* Power off */
    byteVal = SNS_DD_AP3426_ALSPRX_PWR_DOWN;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_MODE_COMMAND, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_MODE_COMMAND err: %d", status);
        return status;
    }

    /* Interrupt Control */
    if ( state->chip.int_gpio == AP3426_INVAILD_GPIO ) {
        /* polling mode disable INT */
        byteVal = SNS_DD_AP3426_ALSPRX_INT_CTRL_POLL;
    } else {
        /* DRI mode */
        byteVal = SNS_DD_AP3426_ALSPRX_INT_CTRL_DRI;
    }
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_INT_CTRL, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_INT_CTRL err: %d", status);
        return status;
    }

    /* Set Prox Interrupt Threshold low 300 */
    setup->prox.thresh_low = SNS_DD_AP3426_PRX_THRESH_FAR;
    byteVal = setup->prox.thresh_low & SNS_AP3426_PRO_TH_MASK;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_LTHL, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_LTHL err: %d", status);
        return status;
    }
    byteVal = setup->prox.thresh_low >> SNS_AP3426_PRO_LTHH_SHIFT;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_LTHH, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_LTHH err: %d", status);
        return status;
    }
    /* Set Prox Threshold Threshold high 600 */
    setup->prox.thresh_high = SNS_DD_AP3426_PRX_THRESH_NEAR;
    byteVal = setup->prox.thresh_high & SNS_AP3426_PRO_TH_MASK;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_HTHL, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_HTHL err: %d", status);
        return status;
    }
    byteVal = setup->prox.thresh_high >> SNS_AP3426_PRO_HTHH_SHIFT;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_HTHH, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_HTHH err: %d", status);
        return status;
    }

    /* Prox Gain */
    byteVal = SNS_AP3426_PRO_GAIN << SNS_AP3426_PRO_GAIN_SHIFT;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_GAIN, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_GAIN err: %d", status);
        return status;
    }

    /* Prox LED Driver */
    byteVal = SNS_AP3426_PRO_LED_DRIVE;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_LED_DRV, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_LED_DRV err: %d", status);
        return status;
    }   

    /* PS Mean Time */
    byteVal = SNS_AP3426_PRO_MEAN_TIME;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_MEAN_TIME, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_MEAN_TIME err: %d", status);
        return status;
    }

    /* Prox Persistence */
    byteVal = SNS_AP3426_PRO_PERSIS;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_PERSIS, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_PX_PERSIS err: %d", status);
        return status;
    }

    /* ALS gain */
    setup->als.gain = SNS_AP3426_ALS_GAIN;
    byteVal = setup->als.gain;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_ALS_GAIN, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_ALS_GAIN err: %d", status);
        return status;
    }

    /* ALS Persis */    
    byteVal = SNS_AP3426_ALS_PERSIS;
    status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_ALS_PERSIS, &byteVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_ALS_PERSIS err: %d", status);
        return status;
    }

    /* Set ALS Threshold */
    setup->als.thresh_low = SNS_AP3426_ALS_THR_LOW_F;
    wordVal.word = setup->als.thresh_low;
    status = sns_dd_ap3426_write_word(state->port_handle, AP3426_ALS_LTHL, &wordVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_ALS_LTHL err: %d", status);
        return status;
    }
    setup->als.thresh_high = SNS_AP3426_ALS_THR_HIG_F;
    wordVal.word = setup->als.thresh_high;
    status = sns_dd_ap3426_write_word(state->port_handle, AP3426_ALS_HTHL, &wordVal);
    if ( status != SNS_DDF_SUCCESS ) {
        DDF_MSG_1(MED, "sns_dd_ap3426_update_registers, write AP3426_ALS_HTHL err: %d", status);
        return status;
    }

    DDF_MSG_0(MED, "ap3426_update_registers() end");

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_prx_query

DESCRIPTION   prox query
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_prx_query
(
  sns_dd_ap3426_alsprx_state_t* dd_ptr,
  sns_ddf_memhandler_s*  memhandler,
  sns_ddf_attribute_e    attrib,
  void**                 value,
  uint32_t*              num_elems
)
{
  uint16_t*             thresh_ptr;
  uint16_t*             accu_ptr;
  sns_ddf_power_info_s* power_ptr;
  uint32_t*             odr_ptr;

  sns_ddf_resolution_adc_s* resp;

  switch ( attrib )
  {
    case SNS_DDF_ATTRIB_RANGE:
    {
      sns_ddf_range_s *device_ranges;
      if( (*value = sns_ddf_memhandler_malloc_ex(memhandler,
                      sizeof(sns_ddf_range_s), dd_ptr->smgr_handle))  == NULL)
      {
          return SNS_DDF_ENOMEM;
      }
      device_ranges = *value;
      *num_elems = 1;
      device_ranges->min = FX_FLTTOFIX_Q16(0);
      device_ranges->max = SNS_DD_AP3426_PRX_RES; /* 5 cm = 0.05m */
      break;
    }

    case SNS_DDF_ATTRIB_RESOLUTION_ADC:
    {
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_adc_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES_ADC,
                                    //SNS_DD_ALSPRX_PRX_BITS);
      resp->bit_len  = SNS_DD_AP3426_ALSPRX_PRX_BITS;
      resp->max_freq = SNS_DD_AP3426_ALSPRX_PRX_FREQ;
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_RESOLUTION:
    {
      sns_ddf_resolution_t* resp;
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;

      *resp = SNS_DD_AP3426_PRX_RES;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES,
                                    //*resp);
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_POWER_INFO:
      if ( (power_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_power_info_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      power_ptr->active_current = SNS_DD_AP3426_PRX_PWR;
      power_ptr->lowpower_current = SNS_DD_AP3426_ALSPRX_LO_PWR;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_PWR,
                                    //power_ptr->active_current); // add power_ptr->lowpower_current to F3
      *(sns_ddf_power_info_s **)value = power_ptr;
      break;

    case SNS_DDF_ATTRIB_ACCURACY:
      if ( (accu_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint16_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_ACC,
                                    //SNS_DD_PRX_ACCURACY);
      *accu_ptr = SNS_DD_AP3426_PRX_ACCURACY;
      *(uint16_t **)value = accu_ptr;
      break;

    case SNS_DDF_ATTRIB_THRESHOLD:
      if ( (thresh_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint16_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      *thresh_ptr = SNS_DD_AP3426_PRX_THRESH_NEAR; 
      *(uint16_t **)value = thresh_ptr; 
      DDF_MSG_0(HIGH, "get_attrib threshold");
      
      break;

    case SNS_DDF_ATTRIB_ODR:
      if ( (odr_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint32_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      *odr_ptr   = dd_ptr->chip.setup.prox.odr;
      *(uint32_t **)value = odr_ptr;
      DDF_MSG_1(HIGH, "AP3426 get_attrib prx odr: %d",  *odr_ptr);
      break;

    case SNS_DDF_ATTRIB_SUPPORTED_ODR_LIST:
    {
      uint32_t *odr_list;

      if((odr_list = (uint32_t *)sns_ddf_memhandler_malloc_ex(
              memhandler,
              1 * sizeof(uint32_t) , dd_ptr->smgr_handle)) == NULL)
      {
        return SNS_DDF_ENOMEM;
      }
      odr_list[0] = SNS_DD_AP3426_ALSPRX_PRX_FREQ;
      *value = (uint32_t *)odr_list;
      *num_elems = 1;
    }
    break;

    default:
      //SNS_PRINTF_STRING_ID_HIGH_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                  //DBG_DD_ALSPRX_GET_ATTRIB_ERR,
                                  //attrib);
      return SNS_DDF_EINVALID_ATTR;
  }
  return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_als_query

DESCRIPTION   als query
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_als_query
(
  sns_dd_ap3426_alsprx_state_t* dd_ptr,
  sns_ddf_memhandler_s*  memhandler,
  sns_ddf_attribute_e    attrib,
  void**                 value,
  uint32_t*              num_elems
)
{
  sns_ddf_power_info_s* power_ptr;
  uint16_t*             accu_ptr;
  uint32_t*             odr_ptr;

  sns_ddf_resolution_adc_s* resp;

  switch ( attrib )
  {
    case SNS_DDF_ATTRIB_RANGE:
    {
      sns_ddf_range_s *device_ranges;
      if( (*value = sns_ddf_memhandler_malloc_ex(memhandler,
                      sizeof(sns_ddf_range_s), dd_ptr->smgr_handle))  == NULL)
      {
          return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr range");
      device_ranges = *value;
      *num_elems = 1;
      device_ranges->min = FX_FLTTOFIX_Q16(0.01);
      device_ranges->max = FX_FLTTOFIX_Q16(10000);
      break;
    }
    case SNS_DDF_ATTRIB_RESOLUTION_ADC:
    {
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_adc_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr resolution ADC");
      *num_elems = 1;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES_ADC,
                                    //SNS_DD_ALSPRX_ALS_BITS);
      resp->bit_len = SNS_DD_AP3426_ALSPRX_ALS_BITS;
      resp->max_freq = SNS_DD_AP3426_ALSPRX_ALS_FREQ;
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_RESOLUTION:
    {
      sns_ddf_resolution_t* resp;
      if ( (resp = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_resolution_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      DDF_MSG_0(HIGH, "in get_attr resolution");
      *num_elems = 1;

      *resp = SNS_DD_AP3426_ALS_RES;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_RES,
                                    //*resp);
      *value = resp;
      break;
    }

    case SNS_DDF_ATTRIB_POWER_INFO:
      if ( (power_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_power_info_s), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      power_ptr->active_current   = SNS_DD_AP3426_ALS_PWR;
      power_ptr->lowpower_current = SNS_DD_AP3426_ALSPRX_LO_PWR;
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_PWR,
                                    //power_ptr->active_current); // add power_ptr->lowpower_current to F3
      *(sns_ddf_power_info_s **)value = power_ptr;
      break;

    case SNS_DDF_ATTRIB_ACCURACY:
      if ( (accu_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint16_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      dd_ptr->sns_dd_als_db.accuracy = dd_ptr->sns_dd_als_db.last_mlux / SNS_DD_AP3426_ALS_ACCURACY_DIV_FACTOR;
      if ( dd_ptr->sns_dd_als_db.accuracy < SNS_DD_AP3426_ALS_HALF_LUX )
      {
        dd_ptr->sns_dd_als_db.accuracy = SNS_DD_AP3426_ALS_HALF_LUX;
      }
      DDF_MSG_0(HIGH, "accuracy");
      //SNS_PRINTF_STRING_ID_MEDIUM_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_ACC,
                                    //dd_ptr->sns_dd_als_db.accuracy);
      *accu_ptr = dd_ptr->sns_dd_als_db.accuracy;
      *(uint16_t **)value = accu_ptr;
      break;

    case SNS_DDF_ATTRIB_ODR:
      if ( (odr_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(uint32_t), dd_ptr->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      *num_elems = 1;
      *odr_ptr   = dd_ptr->chip.setup.als.odr;
      *(uint32_t **)value = odr_ptr;
      DDF_MSG_1(HIGH, "AP3426 get_attrib als odr: %d",  *odr_ptr);
      break;

    case SNS_DDF_ATTRIB_SUPPORTED_ODR_LIST:
    {
      uint32_t *odr_list;

      if((odr_list = (uint32_t *)sns_ddf_memhandler_malloc_ex(
              memhandler,
              1 * sizeof(uint32_t) , dd_ptr->smgr_handle)) == NULL)
      {
        return SNS_DDF_ENOMEM;
      }
      odr_list[0] = SNS_DD_AP3426_ALSPRX_ALS_FREQ;
      *value = (uint32_t *)odr_list;
      *num_elems = 1;
    }
    break;

    default:
      //SNS_PRINTF_STRING_ID_HIGH_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                  //DBG_DD_ALSPRX_GET_ATTRIB_ERR,
                                  //attrib);
      return SNS_DDF_EINVALID_ATTR;
  }
  return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_common_init

DESCRIPTION   initializes als and ps
===========================================================================*/
void sns_dd_alsprx_ap3426_common_init ( sns_dd_ap3426_alsprx_state_t* dd_ptr )
{
  dd_ptr->dropped_logs = 0;
  dd_ptr->sns_dd_alsprx_common_db.state = SNS_DD_AP3426_DEV_STOPPED;

  if (dd_ptr->sns_dd_alsprx_common_db.nv_source == SNS_DD_AP3426_ALSPRX_NV_SOURCE_DEFAULT)
  {
    //SNS_PRINTF_STRING_ID_LOW_0(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                               //DBG_DD_ALSPRX_NV_USE_DRIVER_DEFAULTS);
  }
}
/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_init_data_struct

DESCRIPTION   initialize data structures for ALS and PRX
===========================================================================*/
void sns_dd_alsprx_ap3426_init_data_struct
(
  sns_dd_ap3426_alsprx_state_t* dd_ptr
)
{
  /* Initialize the common database */
  sns_dd_alsprx_ap3426_common_init(dd_ptr);
}
/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_set_powerstate

DESCRIPTION   set power state
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_set_powerstate
(
  sns_dd_ap3426_alsprx_state_t* state,
  sns_ddf_powerstate_e   op_mode,
  sns_ddf_sensor_e       sensor,
  BOOLEAN                init_data
)
{
  uint8_t          i2c_data;
  sns_ddf_status_e status = SNS_DDF_SUCCESS;

        DDF_MSG_3(MED, "ap3426_set_powerstate() var1: %d op_mode %d, sensor: %d", 1126, op_mode, sensor);
  if ( op_mode == SNS_DDF_POWERSTATE_ACTIVE )
  {
      if(sensor == SNS_DDF_SENSOR_PROXIMITY)
      {
          DDF_MSG_1(MED, "ap3426_set_powerstate() ONLY PROX", 1126);
          state->chip.setup.reg.enable |= AP3426x_ENABLE_MSK_PEN;
      }
      else if(sensor == SNS_DDF_SENSOR_AMBIENT)
      {
          DDF_MSG_1(MED, "ap3426_set_powerstate() ONLY ALS", 1126);
          state->chip.setup.reg.enable |= AP3426x_ENABLE_MSK_AEN;
      }
      else if(sensor == SNS_DDF_SENSOR__ALL)
      {
          state->chip.setup.reg.enable |= AP3426x_ENABLE_MSK_PEN;
          state->chip.setup.reg.enable |= AP3426x_ENABLE_MSK_AEN;
          DDF_MSG_1(MED, "ap3426_set_powerstate() var1: %d", 5555);
      }

      i2c_data = state->chip.setup.reg.enable;
      status   = sns_dd_ap3426_write_byte(state->port_handle, AP3426_MODE_COMMAND, &i2c_data);
      if ( status != SNS_DDF_SUCCESS ) {
          DDF_MSG_2(MED, "ap3426_set_powerstate() var1: %d, enable power err: %d", 1127, status);
          return status;
      }

      DDF_MSG_2(MED, "ap3426_set_powerstate() var1: %d var2: %d", 1128, i2c_data);
  }
  else /* when op_mode is SNS_DDF_POWERSTATE_LOWPOWER */
  {
    if (init_data == TRUE)
    {
      /* cleanup */
      sns_dd_alsprx_ap3426_init_data_struct(state);
    }

    i2c_data = SNS_DD_AP3426_ALSPRX_PWR_DOWN;
    status   = sns_dd_ap3426_write_byte(state->port_handle, AP3426_MODE_COMMAND, &i2c_data);
    if ( status != SNS_DDF_SUCCESS )
    {
      return status;
    }
    DDF_MSG_2(MED, "ap3426_set_powerstate() var1: %d var2: %d", 1130, i2c_data);

    state->chip.setup.prox.odr = 0;
    state->chip.setup.als.odr  = 0;
    state->chip.irq_regted = 0;
  }

  state->sns_dd_alsprx_common_db.pwr_mode = op_mode;
  state->sns_dd_alsprx_common_db.state    = SNS_DD_AP3426_DEV_CONFIGURED;
  return status;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_set_reg_values_from_nv

DESCRIPTION   set default parameter from nv
===========================================================================*/
void sns_dd_ap3426_set_reg_values_from_nv(sns_dd_ap3426_alsprx_state_t* state)
{
    if(state->sns_dd_alsprx_common_db.nv_db.ALS_thresh_high != 0)
    {    
        state->chip.setup.als.thresh_high = state->sns_dd_alsprx_common_db.nv_db.ALS_thresh_high;
        DDF_MSG_1(MED, "ALS High written %d", state->chip.setup.als.thresh_high);
    }

    if(state->sns_dd_alsprx_common_db.nv_db.ALS_thresh_low != 0)
    {    
        state->chip.setup.als.thresh_low = state->sns_dd_alsprx_common_db.nv_db.ALS_thresh_low;
        DDF_MSG_1(MED, "ALS Low Written %d", state->chip.setup.als.thresh_low);
    }

    if(state->sns_dd_alsprx_common_db.nv_db.dc_offset != 0)
    {
        state->chip.setup.prox.offset_l = state->sns_dd_alsprx_common_db.nv_db.dc_offset & 0xFF;
        state->chip.setup.prox.offset_h = state->sns_dd_alsprx_common_db.nv_db.dc_offset >> 8;
        DDF_MSG_2(MED, "AP3426 read Cali_reg from nv: l: %d h: %d" , state->chip.setup.prox.offset_l, state->chip.setup.prox.offset_h);
    }
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_init

DESCRIPTION   init
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_init(
                                            sns_ddf_handle_t*        dd_ptr,
                                            sns_ddf_handle_t         smgr_handle,
                                            sns_ddf_nv_params_s*     reg_params,
                                            sns_ddf_device_access_s  device_info[],
                                            uint32_t                 num_devices,
                                            sns_ddf_memhandler_s*    memhandler,
                                            sns_ddf_sensor_e*        sensors[],
                                            uint32_t*                num_sensors
                                            )
{
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;
    sns_dd_ap3426_alsprx_state_t* ptr;

    DDF_MSG_0(HIGH, "AP3426 Init start");

#if AP3426_ENABLE_UIMG_SUPPORT
    sns_ddf_smgr_set_uimg_refac(smgr_handle);
#endif

    if ( (dd_ptr == NULL)      ||
         (num_sensors == NULL) ||
         (sensors == NULL) )
    {
        DDF_MSG_0(HIGH, "invalid param");
        return SNS_DDF_EINVALID_PARAM;
    }

    status = sns_ddf_malloc_ex((void**)&ptr, sizeof(sns_dd_ap3426_alsprx_state_t), smgr_handle);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(MED, "AP3426 Init, memory alloc ap3426_alsprx_state_t err: %d", status);
        return status;
    }

    status = sns_ddf_malloc_ex((void **) &ptr->s_data, 3 * sizeof(sns_ddf_sensor_data_s), smgr_handle);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "AP3426 Init, memory alloc s_data err: %d", status);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_ENOMEM;
    }

    status = sns_ddf_malloc_ex((void **) &ptr->samples, 3 * sizeof(sns_ddf_sensor_sample_s), smgr_handle);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "AP3426 Init, memory alloc samples err: %d", status);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);     
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return SNS_DDF_ENOMEM;
    }

    ptr->smgr_handle     = smgr_handle;
    ptr->int_gpio        = device_info->first_gpio;
    ptr->chip.int_gpio   = ptr->int_gpio;
    ptr->chip.irq_als_en = 0;
    ptr->chip.irq_prx_en = 0;
    ptr->chip.irq_regted = 0;
    ptr->als_auto_gain_delay_time = sns_ddf_convert_usec_to_tick(AP3426_ALS_AUTO_GAIN_USDELAY);
    
    status = sns_ddf_open_port(&ptr->port_handle, &device_info->port_config);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(HIGH, "AP3426 Init, port_handle err: %d", status);        
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex(ptr->samples, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return status;
    }

    device_info->port_config.bus_config.i2c->dev_type = SNS_DDF_I2C_DEVICE_DEFAULT;
    status = sns_ddf_open_port(&ptr->ZBW_port_handle, &device_info->port_config);    

    /* Init timer object */
    if ((status = sns_ddf_timer_init(
            &ptr->sns_dd_tmr_obj,
            (sns_ddf_handle_t)ptr,            
            &AP3426_DD_IF,
            ptr,
            true)) != SNS_DDF_SUCCESS)
    {
       DDF_MSG_1(HIGH, "AP3426 Init, timer init err: %d", status);
       sns_ddf_close_port(ptr->port_handle);
       sns_ddf_close_port(ptr->ZBW_port_handle);
       sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
       sns_ddf_mfree_ex(ptr->samples, smgr_handle);
       sns_ddf_mfree_ex((void *)ptr , smgr_handle);
       return SNS_DDF_EFAIL;
    }


    /* Fill out output parameters*/
    *num_sensors = SNS_DD_AP3426_ALSPRX_NUM_SENSOR_TYPES;
    *sensors     = (sns_ddf_sensor_e *)sns_dd_ap3426_alsprx_sensor_types;
    *dd_ptr      = (sns_ddf_handle_t) ptr;

     DDF_MSG_0(HIGH, "past irq and timer enable");
     ptr->nv_db_size = 0;
     bool data_from_registry = false;     

    /* Recall NV parameters, if any*/
    if(reg_params)
    {
        DDF_MSG_0(HIGH, "ap3426_init yes registry params available");
        if ( (reg_params->status == SNS_DDF_SUCCESS) &&
             (reg_params->data_len >= sizeof(sns_dd_ap3426_nv_db_type)))
        {
            DDF_MSG_0(HIGH, "registry params match nv db type");
            sns_dd_ap3426_nv_db_type *nv_ptr = (sns_dd_ap3426_nv_db_type *)reg_params->data;
            if ( (nv_ptr->lsc_num == AP3426_LSC_MAGIC_NUM) &&
                 (nv_ptr->drv_ID          == AP3426_LSC_DRIVER_ID))
            {
                /* update value from NV parameters*/
                DDF_MSG_0(HIGH, "magic number for AP3426 verified");
                data_from_registry = true;
                ptr->nv_db_size = reg_params->data_len;
                
                //SNS_OS_MEMCOPY(&ptr->sns_dd_alsprx_common_db.nv_db, nv_ptr, sizeof(sns_dd_nv_db_type));
                memcpy(&ptr->sns_dd_alsprx_common_db.nv_db, nv_ptr, sizeof(sns_dd_ap3426_nv_db_type));
                DDF_MSG_0(HIGH, "in registry params past memcopy");                             
                sns_dd_ap3426_set_reg_values_from_nv(ptr);
                DDF_MSG_0(HIGH, "setting defaults from nv");
            }
        }

        if (!data_from_registry) 
        {
            DDF_MSG_0(HIGH, "No registry data");
            ptr->nv_db_size = AP3426_REG_FIXED_NV_SIZE_128;
            ptr->sns_dd_alsprx_common_db.nv_db.visible_ratio = 5;
            ptr->sns_dd_alsprx_common_db.nv_db.ir_ratio = 80;
            ptr->sns_dd_alsprx_common_db.nv_db.dc_offset = 0;
            ptr->sns_dd_alsprx_common_db.nv_db.thresh_near = SNS_DD_AP3426_PRX_THRESH_NEAR;
            ptr->sns_dd_alsprx_common_db.nv_db.thresh_far  = SNS_DD_AP3426_PRX_THRESH_FAR;
            ptr->sns_dd_alsprx_common_db.nv_db.prx_factor  = 1;
            ptr->sns_dd_alsprx_common_db.nv_db.als_factor  = 1;
            ptr->sns_dd_alsprx_common_db.nv_db.lsc_num = (uint32_t)AP3426_LSC_MAGIC_NUM;
            ptr->sns_dd_alsprx_common_db.nv_db.drv_ID  = (uint32_t)AP3426_LSC_DRIVER_ID;
            ptr->sns_dd_alsprx_common_db.nv_db.device = 4;
            ptr->sns_dd_alsprx_common_db.nv_db.ALS_thresh_high = SNS_AP3426_ALS_THR_HIG_F;
            ptr->sns_dd_alsprx_common_db.nv_db.ALS_thresh_low = SNS_AP3426_ALS_THR_LOW_F;

            DDF_MSG_0(HIGH, "Set defaults for registry data");
            /* Writing the same values above to the part*/
            sns_dd_ap3426_set_reg_values_from_nv(ptr);
            DDF_MSG_0(HIGH, "Registry Updated, sending the same values sent to registry to part");

           /*  SNS_OS_MEMZERO(ptr->sns_dd_alsprx_common_db.nv_db.reserved, sizeof(ptr->sns_dd_alsprx_common_db.nv_db.reserved)); (no reserved space here)*/
        }

        if (ptr->sns_dd_alsprx_common_db.nv_db.calibratePhone) 
        {
          
           ptr->calibration_mode = true;
           DDF_MSG_0(HIGH, "setting phone calib  true");
        }
        if ( (reg_params) && (!data_from_registry) ) 
        {
           DDF_MSG_0(HIGH, "before notify");
           sns_ddf_smgr_notify_event(smgr_handle, SNS_DDF_SENSOR_AMBIENT,SNS_DDF_EVENT_UPDATE_REGISTRY_GROUP);
           DDF_MSG_0(HIGH, "after notify");
        }
    }

    status = sns_dd_ap3426_update_registers(ptr);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(HIGH, "AP306E Init update registers err: %d", status);
        sns_ddf_close_port(ptr->port_handle);
        sns_ddf_close_port(ptr->ZBW_port_handle);
        sns_ddf_mfree_ex(ptr->s_data, smgr_handle);
        sns_ddf_mfree_ex(ptr->samples, smgr_handle);
        sns_ddf_mfree_ex((void *)ptr , smgr_handle);
        return status;
    }

    DDF_MSG_0(HIGH, "AP3426 Init end");

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_reset

DESCRIPTION   reset als and ps sensor
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_reset(sns_ddf_handle_t dd_handle)
{
    sns_dd_ap3426_alsprx_state_t* state = (sns_dd_ap3426_alsprx_state_t *) dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;

    DDF_MSG_1(MED, "AP3426 reset var1: %d", 1149);
    /* Reset driver state */    
    state->chip.irq_regted = 0;
    DDF_MSG_1(MED, "AP3426 reset exit: %d", status);
    return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_set_attrib

DESCRIPTION   set attrib
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_set_attrib
(
  sns_ddf_handle_t     dd_handle,
  sns_ddf_sensor_e     sensor,
  sns_ddf_attribute_e  attrib,
  void*                value
)
{
    sns_dd_ap3426_alsprx_state_t *state  = dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;

    DDF_MSG_1(MED, "ap3426_set_attrib() start AP3426 var1: %d", 1153);

  if ( (dd_handle == NULL) ||
       (value == NULL) )
  {
    return SNS_DDF_EINVALID_PARAM;
  }

  switch ( attrib )
  {
  case SNS_DDF_ATTRIB_POWER_STATE:

      /* set power mode */
      DDF_MSG_1(MED, "power mode set %d", 1154);
      DDF_MSG_2(MED, "power mode set value: %d, sensor: %d", *(sns_ddf_powerstate_e *)value, sensor);
      status = sns_dd_alsprx_ap3426_set_powerstate((sns_dd_ap3426_alsprx_state_t *)dd_handle,
                                            *(sns_ddf_powerstate_e *)value,
                                            sensor,
                                            TRUE);
      if(status != SNS_DDF_SUCCESS)
      {
          DDF_MSG_1(MED, "AP3426 var1: %d", 1154);
          return status;
      }

      break;

  case SNS_DDF_ATTRIB_ODR:
          if(sensor == SNS_DDF_SENSOR_PROXIMITY)
          {
            state->chip.setup.prox.odr = *(uint32_t*)value;
          }
          else if(sensor == SNS_DDF_SENSOR_AMBIENT)
          {
            state->chip.setup.als.odr = *(uint32_t*)value;
          }
          else
          {
            return SNS_DDF_EINVALID_ATTR;
          }
      break;

  default:
      return SNS_DDF_EINVALID_PARAM;
  }
        if(state->chip.setup.prox.odr > SNS_DD_AP3426_ALSPRO_INT_ODR && state->chip.setup.als.odr > SNS_DD_AP3426_ALSPRO_F_ODR) {
          state->chip.setup.prox.odr = SNS_DD_AP3426_ALSPRO_INT_ODR;
          DDF_MSG_1(MED, "ap3426_set_attrib() setup.prox.odr = %d", state->chip.setup.prox.odr);
        }
        if(state->chip.setup.als.odr > SNS_DD_AP3426_ALSPRO_INT_ODR) {
          state->chip.setup.als.odr = SNS_DD_AP3426_ALSPRO_INT_ODR;
          DDF_MSG_1(MED, "ap3426_set_attrib() setup.als.odr = %d", state->chip.setup.als.odr);
        }

    DDF_MSG_0(MED, "ap3426_set_attrib() end");

  return(status);
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_get_attrib

DESCRIPTION   sensor get attrib
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_get_attrib
(
  sns_ddf_handle_t      handle,
  sns_ddf_sensor_e      sensor,
  sns_ddf_attribute_e   attrib,
  sns_ddf_memhandler_s* memhandler,
  void**                value,
  uint32_t*             num_elems
)
{
  sns_dd_ap3426_alsprx_state_t* state = (sns_dd_ap3426_alsprx_state_t *)handle;  
  sns_ddf_status_e       status = SNS_DDF_SUCCESS;
  BOOLEAN                generic_attrib = FALSE;
  sns_ddf_registry_group_s  *reg_group;
  uint8_t                   *reg_group_data;
  sns_ddf_driver_info_s *driver_info_ptr;
  sns_ddf_driver_info_s  driver_info_l = {
    "AP3426",  /* name */
    1           /* version */
  };

  sns_ddf_device_info_s *device_info_ptr;
  sns_ddf_device_info_s  device_info_l = {
    "ALSPRX",   /* name */
    "DynaIamge",   /* vendor */
    "AP3426",  /* model */
    1           /* version */
  };

  if ( (handle == NULL)      ||
       (memhandler == NULL) )
  {
    return SNS_DDF_EINVALID_PARAM;
  }

  //SNS_PRINTF_STRING_ID_MEDIUM_2(SNS_DBG_MOD_DSPS_DD_ALSPRX, DBG_DD_ALSPRX_GET_ATTRIB_REQ, sensor,attrib );

  /* check first if the query is for generic attributes */
  switch ( attrib )
  {
    case SNS_DDF_ATTRIB_DRIVER_INFO:
      if ( (driver_info_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_driver_info_s), state->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }      
      *driver_info_ptr = driver_info_l; /* Is memcopy needed because of strings */
      *(sns_ddf_driver_info_s**)value = driver_info_ptr;      
      *num_elems = 1;
      generic_attrib = TRUE;
      break;

    case SNS_DDF_ATTRIB_DEVICE_INFO:
      if ( (device_info_ptr = sns_ddf_memhandler_malloc_ex(memhandler, sizeof(sns_ddf_device_info_s), state->smgr_handle)) == NULL )
      {
        return SNS_DDF_ENOMEM;
      }
      if (sensor == SNS_DDF_SENSOR_AMBIENT) {
        device_info_l.name = "ALS";
      } else if (sensor == SNS_DDF_SENSOR_PROXIMITY) {
        device_info_l.name = "PROX";
      } else {
        return SNS_DDF_EFAIL;
      }
      *device_info_ptr = device_info_l;
      *(sns_ddf_device_info_s**)value = device_info_ptr; /* Is memcopy needed instead? */
      *num_elems = 1;
      generic_attrib = TRUE;
      break;

    case SNS_DDF_ATTRIB_DELAYS:
    {
      sns_ddf_delays_s *device_delay;
      if ((*value = sns_ddf_memhandler_malloc_ex(
              memhandler,
              sizeof(sns_ddf_delays_s) , state->smgr_handle)) == NULL)
      {
        return SNS_DDF_ENOMEM;
      }

      device_delay  = (sns_ddf_delays_s *)(*value);
      *num_elems    = 1;
      /* Value is in int in usec */
      if (sensor == SNS_DDF_SENSOR_AMBIENT) {
          device_delay->time_to_active  = SNS_DD_AP3426_ALSPRX_TIME_ACT;
          device_delay->time_to_data    = SNS_DD_AP3426_ALSPRX_TIME_DATA;
      } else {
          device_delay->time_to_active  = SNS_DD_AP3426_ALSPRX_TIME_ACT;
          device_delay->time_to_data    = SNS_DD_AP3426_ALSPRX_TIME_DATA;
      }
    }
    break;
          
    case SNS_DDF_ATTRIB_REGISTRY_GROUP:
      DDF_MSG_0(HIGH, "in attrib reg group");
      reg_group = sns_ddf_memhandler_malloc_ex(memhandler,sizeof(*reg_group), state->smgr_handle);
      if (reg_group == NULL)
      {      
        DDF_MSG_0(HIGH, "no reg group");
        return SNS_DDF_ENOMEM;
      }

      if (state->nv_db_size == 0) 
      {
        reg_group->group_data = NULL;
        reg_group->size = 0;
        DDF_MSG_0(HIGH, "nv db size 0");
      } 
      else 
      {
        reg_group_data = sns_ddf_memhandler_malloc_ex(memhandler, state->nv_db_size, state->smgr_handle);
        if (reg_group_data == NULL)
        {      
          return SNS_DDF_ENOMEM;
        }
        DDF_MSG_0(HIGH, "at memcopy");
     
        //SNS_OS_MEMCOPY(reg_group_data, &state->sns_dd_alsprx_common_db.nv_db, sizeof(sns_dd_nv_db_type));
        memcpy(reg_group_data, &state->sns_dd_alsprx_common_db.nv_db, sizeof(sns_dd_ap3426_nv_db_type));
        reg_group->group_data = reg_group_data;
        reg_group->size = state->nv_db_size;
        DDF_MSG_0(HIGH, "after memcopy");
      }
      
      *value = reg_group;
      *num_elems = 1;
      generic_attrib = true;
      DDF_MSG_0(HIGH, "out of attrib group");
      return SNS_DDF_SUCCESS;
      
    default:
      /* do nothing */
    break;
  }

  /* The query is not for generic attribute but is for specific data type */
  if ( generic_attrib != TRUE )
  {
    switch ( sensor )
    {
      case SNS_DDF_SENSOR_AMBIENT:
        status = sns_dd_alsprx_ap3426_als_query(state, memhandler, attrib, value, num_elems);
        break;

      case SNS_DDF_SENSOR_PROXIMITY:
        status = sns_dd_alsprx_ap3426_prx_query(state, memhandler, attrib, value, num_elems);
        break;

      default:
        //SNS_PRINTF_STRING_ID_HIGH_1(SNS_DBG_MOD_DSPS_DD_ALSPRX,
                                    //DBG_DD_ALSPRX_GET_ATTRIB_SENSOR_TYPE_ERR,
                                    //sensor);
        status = SNS_DDF_EINVALID_ATTR;
    }
  }

  return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_run_test

DESCRIPTION   sensor self test
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_run_test
( sns_ddf_handle_t  dd_handle,
  sns_ddf_sensor_e  sensor,
  sns_ddf_test_e    test,
  uint32_t*         err
)
{
    sns_dd_ap3426_alsprx_state_t *state  = dd_handle;
    sns_ddf_status_e       status = SNS_DDF_SUCCESS;
    sns_ddf_handle_t       smgr_handle;
    uint8_t readByte;

    DDF_MSG_2(HIGH, "run test: sensor %d test %d", sensor, test);

    if(sensor == SNS_DDF_SENSOR_AMBIENT)
    {
        DDF_MSG_0(HIGH, "in selftest sensor_ambient ");
        if(test == SNS_DDF_TEST_SELF)
        {
                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_INT_CTRL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_INT_CTRL value = 0x%02x",readByte);

                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_GAIN, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_GAIN value = 0x%02x",readByte);

                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_PERSIS, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;              
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_PERSIS value = 0x%02x",readByte);        
            
                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_LTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_LTHL value = 0x%02x",readByte);
            
                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_LTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_LTHH value = 0x%02x",readByte);
                
                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_HTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_HTHL value = 0x%02x",readByte);
                
                status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_HTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;              
                DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_ALS_HTHH value = 0x%02x",readByte);                      
        } else if(test == SNS_DDF_TEST_OEM) {
            DDF_MSG_0(HIGH, "in selftest sensor_ambient OEM test ");
            smgr_handle = state->smgr_handle;

            status = sns_dd_alsprx_ap3426_reset(state);

            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in reset ");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AP3426_ALS_CAL_FAILED_RST);
                return status;
            }
            DDF_MSG_0(HIGH, "after reset"); 

            /*  enable sensor */
                status = sns_dd_alsprx_ap3426_set_powerstate(dd_handle, SNS_DDF_POWERSTATE_ACTIVE ,SNS_DDF_SENSOR__ALL, TRUE);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in set power state -sensor enable ");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AP3426_ALS_CAL_FAILED_ENB);
                return status;
            }

            DDF_MSG_0(HIGH, "out of als scaling");
            status = sns_ddf_smgr_notify_event(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EVENT_UPDATE_REGISTRY_GROUP);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "failed in selftest -no registry notify");
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_AMBIENT, SNS_DDF_EFAIL, SNS_DD_AP3426_ALS_CAL_FAILED_REG);
                return status;
            }
            DDF_MSG_0(HIGH, "successful registry update notify complete");
        } else {
            DDF_MSG_0(HIGH, "NOT OEM test ");
            return SNS_DDF_EINVALID_TEST;
        }
    } else if(sensor == SNS_DDF_SENSOR_PROXIMITY) {
        if(test == SNS_DDF_TEST_SELF) {
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_INT_CTRL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_INT_CTRL value = 0x%02x",readByte);

                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_LED_WTIME, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_LED_WTIME value = 0x%02x",readByte);

                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_GAIN, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_GAIN value = 0x%02x",readByte);   

                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_LED_DRV, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_LED_DRV value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_MEAN_TIME, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_MEAN_TIME value = 0x%02x",readByte);

                    status = sns_dd_ap3426_read_byte(state->port_handle,AP3426_PX_PERSIS, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_PERSIS value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_L, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_CAL_L value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_H, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_CAL_H value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_LTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_LTHL value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_LTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_LTHH value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_HTHL, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_HTHL value = 0x%02x",readByte);
                    
                    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_HTHH, &readByte);
            if( status != SNS_DDF_SUCCESS ) return status;
                    DDF_MSG_1(HIGH, "sns_dd_alsprx_ap3426_run_test read AP3426_PX_HTHH value = 0x%02x",readByte);
        } else if(test == SNS_DDF_TEST_OEM) {
#define ARYSIZE_3 20
            int         i, j;
            uint8_t     Mode, Reg;
            uint8_t     orig_cal_h = 0, orig_cal_l = 0;
            uint16_t    ps_data=0,g_pscali=0,temp_pdata[ARYSIZE_3];

            smgr_handle = state->smgr_handle;
            DDF_MSG_0(HIGH, "prox sensor: calibrator ...");

            /* save current mode */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_MODE_COMMAND, &Mode);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_MODE_COMMAND");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 in mode_reg is: 0x%x" , Mode);

            /* read current offset_l */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_L, &orig_cal_l);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_L");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 in Cali_L_reg is: %d" , orig_cal_l);

            /* read current offset_h */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_H, &orig_cal_h);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_H");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 in Cali_H_reg is: %d" , orig_cal_h);

            /* clear Prox CAL offset and will update to sensor during following reset. */
            state->chip.setup.prox.offset_h = 0;
            state->chip.setup.prox.offset_l = 0;

            /* reset chip */
            status = sns_dd_alsprx_ap3426_reset(state);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed in reset ");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_RST);
                return status;
            }
            /* delay 100ms */            
            sns_ddf_delay(100000);
            DDF_MSG_0(HIGH, "prox cali: after reset"); 

            /* read offset to ensure that CAL register was cleared */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_L, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_L");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 in Cali_L_reg is: %d" , Reg);
            if (Reg != 0) {
                DDF_MSG_0(HIGH, "P-CAL CAL_L is not empty\n");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return SNS_DDF_EFAIL;
            }

            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_H, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_H");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 in Cali_H_reg is: %d" , Reg);
            if (Reg != 0) {
                DDF_MSG_0(HIGH, "P-CAL CAL_H is not empty\n");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return SNS_DDF_EFAIL;
            }

            /*  enable sensor */
            status = sns_dd_alsprx_ap3426_set_powerstate(state, SNS_DDF_POWERSTATE_ACTIVE ,SNS_DDF_SENSOR_PROXIMITY, TRUE);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "prox cali: failed set powerstate");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_ENB);
                return status;
            }

            for(i = 0; i < ARYSIZE_3; i++)
            {
                if((status = sns_dd_ap3426_get_prox_data(state)) != SNS_DDF_SUCCESS) 
                {
                    DDF_MSG_1(HIGH, "prox cali: Get Data prx err5: %d", status);
                    sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                    return status;
                }
                temp_pdata[i] = state->chip.data.prox.data;                
                sns_ddf_delay(25000);
            }

            /* pdata sorting small to big */
            for (i = 0; i < ARYSIZE_3 - 1; i++)
            {
                for (j = i+1; j < ARYSIZE_3; j++)
                {
                    if (temp_pdata[i] > temp_pdata[j])
                    {
                        ps_data=temp_pdata[i];
                        temp_pdata[i]=temp_pdata[j];
                        temp_pdata[j]=ps_data;
                    }
                }
            }

            for (i = 5; i < 15; i++)
            {
                g_pscali = g_pscali + temp_pdata[i];
            }
            g_pscali /= 10;
            DDF_MSG_1(MED, "prox cali: average data: %d", g_pscali);
            DDF_MSG_1(HIGH, "P-CAL avg crosstalk: %d\n", g_pscali);

            if(g_pscali > 100) /* For DVT-2, avg crosstalk should be 45 */
            {
                DDF_MSG_0(HIGH, "prox cali: x-talkt* too big");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_DATA);
                return -11;
            }

            /*  enable sensor */
            status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_MODE_COMMAND, &Mode);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed write AP3426_MODE_COMMAND");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }

            /* write prox calibration data to register */
            Reg = g_pscali & 0xFF;
            status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_CAL_L, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed write AP3426_PX_CAL_L");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }

            Reg = g_pscali >> 8;
            status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_PX_CAL_H, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed write AP3426_PX_CAL_H");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }

            /* read cali offset_l */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_L, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_L");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 111 in Cali_L_reg is: %d" , Reg);

            /* update memory configure */
            state->chip.setup.prox.offset_l = Reg;

            /* read cali offset_h */
            status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_PX_CAL_H, &Reg);
            if( status != SNS_DDF_SUCCESS ) {
                DDF_MSG_0(HIGH, "prox cali: failed read AP3426_PX_CAL_H");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_CHIP);
                return status;
            }
            DDF_MSG_1(MED, "AP3426 111 in Cali_H_reg is: %d" , Reg);

            /* update memory configure */
            state->chip.setup.prox.offset_h = Reg;

            /* update registry group */
            state->sns_dd_alsprx_common_db.nv_db.dc_offset = g_pscali;
            status = sns_ddf_smgr_notify_event(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EVENT_UPDATE_REGISTRY_GROUP);
            if( status != SNS_DDF_SUCCESS )
            {
                DDF_MSG_0(HIGH, "prox cali: failed in selftest -no registry notify");
                sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, SNS_DDF_EFAIL, SNS_DD_AP3426_PROX_CAL_FAILED_REG);
                return status;
            }
            sns_ddf_smgr_notify_test_complete(smgr_handle, SNS_DDF_SENSOR_PROXIMITY, (sns_ddf_status_e)SNS_DD_AP3426_SUCCESS, 0);
            DDF_MSG_0(HIGH, "prox cali: successful registry update notify complete");
        } else {
            DDF_MSG_0(HIGH, "NOT correct test ");
            return SNS_DDF_EINVALID_TEST;
        }
    } else {
        DDF_MSG_0(HIGH, "NOT CORRECT Sensor");
        return SNS_DDF_EINVALID_TEST;
    }

    return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_probe

DESCRIPTION   sensor probe
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_probe
(
 sns_ddf_device_access_s* device_info,
 sns_ddf_memhandler_s*    memhandler,
 uint32_t*                num_sensors,
 sns_ddf_sensor_e**       sensors
)
{
  sns_ddf_status_e status;
  sns_ddf_handle_t port_handle;
  uint8_t i2c_buff;
  uint8_t bytes_read;

  *num_sensors = 0;
  *sensors = NULL;

  status = sns_ddf_open_port(&port_handle, &(device_info->port_config));
  if(status != SNS_DDF_SUCCESS)
  {
    return status;
  }

  /* Read & Verify Device ID */
  status = sns_ddf_read_port( port_handle,
                              AP3426_MODE_COMMAND,
                              &i2c_buff,
                              1,
                              &bytes_read );
                             
  if(status != SNS_DDF_SUCCESS)
  {
    sns_ddf_close_port( port_handle );
    return status;
  }

  *num_sensors = SNS_DD_AP3426_ALSPRX_NUM_SENSOR_TYPES;
  *sensors = sns_ddf_memhandler_malloc( memhandler,
                                        sizeof(sns_ddf_sensor_e) * *num_sensors );
  if( *sensors != NULL )
  {
    (*sensors)[0] = SNS_DDF_SENSOR_PROXIMITY;
    (*sensors)[1] = SNS_DDF_SENSOR_AMBIENT;
    status = SNS_DDF_SUCCESS;
  } else {
    status = SNS_DDF_ENOMEM;
  }
    sns_ddf_close_port( port_handle );
  return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_enable_sched_data

DESCRIPTION   enable schedule data
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_enable_sched_data(sns_ddf_handle_t dd_handle,
                                                         sns_ddf_sensor_e sensor,
                                                         bool             enable
                                                         )
{
    sns_dd_ap3426_alsprx_state_t* state = dd_handle;
    uint8_t byteVal;
    sns_ddf_status_e status = SNS_DDF_SUCCESS;
    uint8_t count = 0;

    DDF_MSG_3(HIGH, "ap3426_enable_sched_data enable: %d, irq_regted: %d, sensor: %d", enable, state->chip.irq_regted, sensor);

    /* Input sanity check */
    if (state == NULL)
    {
        DDF_MSG_0(HIGH, "ap3426_enable_sched_data state is NULL err");
        return SNS_DDF_EINVALID_PARAM;
    }

    /* check parameter - sensor */
    if ((sensor != SNS_DDF_SENSOR_AMBIENT) && (sensor != SNS_DDF_SENSOR_PROXIMITY))
    {
        DDF_MSG_1(HIGH, "ap3426_enable_sched_data sensor type err, sensor: %d", sensor);
        return SNS_DDF_EINVALID_PARAM;
    }

    if (sensor == SNS_DDF_SENSOR_AMBIENT) {
        state->chip.irq_als_en = enable;        
    } else {
        state->chip.irq_prx_en = enable;
    }

    DDF_MSG_2(HIGH, "ap3426_enable_sched_data sensor als_irq_en: %d, prx_irq_en: %d", state->chip.irq_als_en, state->chip.irq_prx_en);

    /* Check if the platform supports interrupts. */
    if( !sns_ddf_signal_irq_enabled() || (state->int_gpio == 0))
    {
        DDF_MSG_1(HIGH, "ap3426_enable_sched_data irq(%d) is not supported", state->int_gpio);
        return SNS_DDF_EFAIL;
    }

    if (enable && !state->chip.irq_regted) {
        byteVal = SNS_DD_AP3426_CLR_ALL_INT;
        while ((status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_INT_STATUS, &byteVal)) != SNS_DDF_SUCCESS) {           
            if (count == 10) {
                DDF_MSG_2(HIGH, "ap3426_enable_sched_data clear interrupt err: %d, count: %d", status, count);
                count = 0;
                return status;
            }
            count++;
            sns_ddf_delay(5000);
        }
        
        while ((status = sns_ddf_signal_register(state->int_gpio, dd_handle,
                                         &AP3426_DD_IF,                                         
                                         SNS_DDF_SIGNAL_IRQ_FALLING)) != SNS_DDF_SUCCESS) { 
            if (count == 5) {
                DDF_MSG_3(HIGH, "ap3426_enable_sched_data irq(%d) registry err: %d, count: %d", state->int_gpio, status, count);
                count = 0;
                return status;
            }           
            count++;
            sns_ddf_delay(5000);
        }
        if (status == SNS_DDF_SUCCESS) {
            DDF_MSG_0(HIGH, "ap3426_enable_sched_data registry success");
            state->chip.irq_regted = 1;             
        }
    } else {
        if (state->chip.irq_regted && !state->chip.irq_als_en && !state->chip.irq_prx_en) {
            status = sns_ddf_signal_deregister(state->int_gpio);
            if (status != SNS_DDF_SUCCESS) {
                DDF_MSG_2(HIGH, "ap3426_enable_sched_data irq(%d) de-registry err: %d", state->int_gpio, status);            
            } else {
                DDF_MSG_1(HIGH, "ap3426_enable_sched_data irq(%d) de-registry success", state->int_gpio);
                state->chip.irq_regted = 0;
            }
            byteVal = SNS_DD_AP3426_CLR_ALL_INT;
            sns_dd_ap3426_write_byte(state->port_handle, AP3426_INT_STATUS, &byteVal);
        } else {
            DDF_MSG_0(HIGH, "ap3426_enable_sched_data not need to de-registry");
        }       
    }    

    return status;
}

