/*==============================================================================

    S E N S O R S   AMBIENT LIGHT AND PROXIMITY  D R I V E R

DESCRIPTION

   Implements the driver for the ap3426 ALS(Light) and Proximity Sensor

Copyright (c) 2015 by QUALCOMM Technologies, Incorporated.  All Rights Reserved.
Qualcomm Technologies Proprietary and Confidential
==============================================================================*/
/*==============================================================================

                      EDIT HISTORY FOR FILE

  This section contains comments describing changes made to the module.
  Notice that changes are listed in reverse chronological order.

$Header: //source/qcom/qct/core/pkg/dsps/rel/2.2/adsp_proc/Sensors/dd/qcom/src/sns_dd_alsprx_ap3426_priv.h#1 $


***************************************************************************
* REVISON HISTORY
* VERSION   DATE(MM/DD/YY)  WHO     DESCRIPTION
* 3.0       03/10/2017      LO      Refactoring scenario of als auto gain
* 2.0       06/18/2016      LO      Add a compiled flag for enable/disable uImage
* 1.0       12/22/2015      LO      Initial version of driver for AP3426
***************************************************************************
==============================================================================*/

#ifndef _SNSAP3426_H
#define _SNSAP3426_H

#include "fixed_point.h"
#include "sns_ddf_util.h"
#include <stdbool.h>
#include <stdio.h>
#include "sns_ddf_attrib.h"
#include "sns_ddf_common.h"
#include "sns_ddf_comm.h"
#include "sns_ddf_driver_if.h"
#include "sns_ddf_smgr_if.h"
#include "sns_ddf_util.h"
#include "sns_ddf_memhandler.h"
#include "sns_ddf_signal.h"


#define AP3426_ENABLE_UIMG_SUPPORT 0
#if AP3426_ENABLE_UIMG_SUPPORT
#else
#define sns_ddf_malloc_ex(ptr, size, smgr_handle)                               sns_ddf_malloc(ptr, size)
#define sns_ddf_mfree_ex(ptr, smgr_handle)                                      sns_ddf_mfree(ptr)
#define sns_ddf_memhandler_malloc_ex(memhandler, size, smgr_handle)             sns_ddf_memhandler_malloc(memhandler, size)
#define sns_ddf_smgr_is_in_uimage()     false
#endif /* End of AP3426_ENABLE_UIMG_SUPPORT */

sns_ddf_driver_if_s sns_alsprx_ap3426_driver_fn_list;
#define AP3426_DD_IF sns_alsprx_ap3426_driver_fn_list

#define AP3426_REG_FIXED_NV_SIZE_128          128
#define AP3426_ALS_AUTO_GAIN_EN               0
#define AP3426_ALS_AUTO_GAIN_MAX_DATA         65535
#define AP3426_ALS_AUTO_GAIN_HTHRES           ((3*AP3426_ALS_AUTO_GAIN_MAX_DATA)/4)
#define AP3426_ALS_AUTO_GAIN_LTHRES           ((2*AP3426_ALS_AUTO_GAIN_MAX_DATA)/16)
#define AP3426_ALS_AUTO_GAIN_USDELAY          200000 // mirco sec

#define AP3426_DD_HANDLE_ALIGN     3

#define AP3426_LSC_MAGIC_NUM   50
#define AP3426_LSC_DRIVER_ID   1
#define AP3426_INVAILD_GPIO    0xffff

#define AP3426_STA_PRX_INTR            0x02
#define AP3426_STA_ALS_INTR            0x01
#define AP3426_CLS_PS_INTR             ~AP3426_STA_PRX_INTR
#define AP3426_CLS_ALS_INTR            ~AP3426_STA_ALS_INTR
#define AP3426x_ENABLE_MSK_PEN         0x02
#define AP3426x_ENABLE_MSK_AEN         0x01
#define AP3426x_ALSPRX_PWR_DOWN        0x00

#define SNS_DD_AP3426_ALSPRX_NUM_SENSOR_TYPES  2
#define SNS_DD_AP3426_PRX_THRESH_NEAR          600
#define SNS_DD_AP3426_PRX_THRESH_FAR           300
#define SNS_DD_AP3426_ALS_PWR                  170
#define SNS_DD_AP3426_ALS_RES                  FX_FLTTOFIX_Q16(1.0)
#define SNS_DD_AP3426_ALSPRX_LO_PWR            2
#define SNS_DD_AP3426_PRX_PWR                  325
#define SNS_DD_AP3426_PRX_RES                  FX_FLTTOFIX_Q16(0.05)
#define SNS_DD_AP3426_PRX_ACCURACY             40
#define SNS_DD_AP3426_ALS_ACCURACY_DIV_FACTOR  4
#define SNS_DD_AP3426_ALS_HALF_LUX             500
#define SNS_DD_AP3426_ALSPRX_PRX_BITS          16
#define SNS_DD_AP3426_ALSPRX_PRX_FREQ          80
#define SNS_DD_AP3426_ALSPRX_ALS_BITS          16
#define SNS_DD_AP3426_ALSPRX_ALS_FREQ          10
#define SNS_DD_AP3426_ALSPRX_TIME_ACT          300000
#define SNS_DD_AP3426_ALSPRX_TIME_DATA         0


#define AP3426_RANGE_A         0x30
#define AP3426_RANGE_B         0x20
#define AP3426_RANGE_C         0x10
#define AP3426_RANGE_D         0x00

#define AP3426_RANGE_A_RATION  0.0078   // 512lux  / 65535
#define AP3426_RANGE_B_RATION  0.03125  // 2048lux / 65535
#define AP3426_RANGE_C_RATION  0.125    // 8192lux / 65535
#define AP3426_RANGE_D_RATION  0.5      // 32768lux / 65535


#define SNS_DD_AP3426_CLR_ALL_INT           0x00
#define SNS_DD_AP3426_ALSPRX_PWR_DOWN       0x00
#define SNS_DD_AP3426_ALSPRX_INT_CTRL_POLL  0x00
#define SNS_DD_AP3426_ALSPRX_INT_CTRL_DRI   0x89
#define SNS_DD_AP3426_ALSPRX_WAITE          0x04
#define SNS_DD_AP3426_ALSPRO_F_ODR          0
#define SNS_DD_AP3426_ALSPRO_INT_ODR        SNS_DD_AP3426_ALSPRX_ALS_FREQ

#define SNS_AP3426_ALS_THR_LOW_F            0xffff
#define SNS_AP3426_ALS_THR_HIG_F            0x0000
#define SNS_AP3426_ALS_INT_CNT_L            27 // depend on user case
#define SNS_AP3426_ALS_INT_CNT_H            (SNS_AP3426_ALS_THR_LOW_F-SNS_AP3426_ALS_INT_CNT_L)
#define SNS_AP3426_ALS_CAL_TARGET           0x40
#define SNS_AP3426_RANGE_MASK               0x30
#define SNS_AP3426_ALS_GAIN                 0x00
#define SNS_AP3426_ALS_GAIN_MASK            0x30
#define SNS_AP3426_ALS_GAIN_SHIFT           4
#define SNS_AP3426_ALS_PERSIS               0x01

#define SNS_AP3426_PRO_OBJ_MASK             0x10
#define SNS_AP3426_PRO_OBJ_SHIFT            4
#define SNS_AP3426_PRO_TH_MASK              0xff
#define SNS_AP3426_PRO_LTHH_SHIFT           8
#define SNS_AP3426_PRO_HTHH_SHIFT           8
#define SNS_AP3426_PRO_PERSIS               0x02
#define SNS_AP3426_PRO_LED_DRIVE            0x03
#define SNS_AP3426_PRO_LED_PLDR_SHIFT       6
#define SNS_AP3426_PRO_GAIN                 0x01
#define SNS_AP3426_PRO_GAIN_SHIFT           2
#define SNS_AP3426_PRO_CTRL_PLPUC           0x01
#define SNS_AP3426_PRO_CTRL_PLUC_SHIFT      4
#define SNS_AP3426_PRO_MEAN_TIME            0x00
#define SNS_AP3426_PRO_DATA_MASK            0x03ff

//system registers
#define AP3426_MODE_COMMAND     0x00
#define AP3426_INT_STATUS       0x01
#define AP3426_INT_CLEAR        0x01
#define AP3426_INT_CTRL         0x02
#define AP3426_WAITE_TIME       0x06
#define AP3426_INT_CLEAR_MASK   0x7c
#define AP3426_PX_ADC_LSB       0x0e
#define AP3426_PX_ADC_MSB       0x0f

//als registers
#define AP3426_ALS_GAIN     0x10
#define AP3426_ALS_LDATA_L  0x0c
#define AP3426_ALS_LDATA_H  0x0d
#define AP3426_ALS_LTHL     0x1a
#define AP3426_ALS_LTHH     0x1b
#define AP3426_ALS_HTHL     0x1c
#define AP3426_ALS_HTHH     0x1d
#define AP3426_ALS_PERSIS   0x14

//ps registers
#define AP3426_PX_LED_CTRL   0x10
#define AP3426_PX_INT_FORM   0x02
#define AP3426_PX_LED_WTIME  0x06
#define AP3426_PX_GAIN       0x20
#define AP3426_PX_LED_DRV    0x21
#define AP3426_PX_MEAN_TIME  0x23
#define AP3426_PX_PERSIS     0x26
#define AP3426_PX_CAL_L      0x28
#define AP3426_PX_CAL_H      0x29
#define AP3426_PX_LTHL       0x2a
#define AP3426_PX_LTHH       0x2b
#define AP3426_PX_HTHL       0x2c
#define AP3426_PX_HTHH       0x2d

typedef enum {
    AP3426_IRQ_TYPE,
    AP3426_TIMER_TYPE
} AP3426_SIG_TYPE;

typedef union {
    uint16_t   word;
    uint8_t    byte[2];
} ap3426_uword_u;

typedef struct {
    uint8_t  gain;  
    uint16_t cal_target;
    uint16_t thresh_low;
    uint16_t thresh_high;
    uint32_t odr;
    bool     enabled;
    uint8_t  persis;
} ap3426_als_setup;

typedef struct {
    uint8_t  gain;
    uint8_t  led_ctrl;
    uint16_t thresh_low;
    uint16_t thresh_high;
    uint8_t  persis;
    uint8_t  control;
    uint32_t odr;
    bool     enabled;
    uint8_t  offset_l;
    uint8_t  offset_h;
    uint8_t  time;
} ap3426_prox_setup;

typedef struct {
    uint8_t  enable;
    uint8_t  config;
    uint8_t  control;
    uint8_t  int_ctrl;
} ap3426_alsprx_reg;

typedef struct  {
    int8_t  gain;
    int32_t data;
    int32_t auto_gain_data;
    int64_t lux;
    int64_t mlux;
    bool     lux_ok;
} ap3426_als_info;

/* proximity data */
typedef struct  {
    uint16_t data;
    int16_t  event;
} ap3426_prox_info;

typedef struct {
    ap3426_als_info      als;    
    ap3426_prox_info     prox;
} ap3426_lsc_cur_info;

typedef struct  {
    ap3426_als_setup   als;
    ap3426_prox_setup  prox;
    ap3426_alsprx_reg  reg;
} ap3426_lsc_setup;

typedef struct {
    ap3426_lsc_cur_info       data;
    ap3426_lsc_setup          setup;
    uint32_t                  int_gpio;
    uint8_t                   irq_regted;
    uint8_t                   irq_als_en;
    uint8_t                   irq_prx_en;
} ap3426_chip;

typedef enum
{
    SNS_DD_AP3426_SUCCESS = 0,
    SNS_DD_AP3426_ALS_CAL_FAILED_LUX = 1, /*ALS CAL error - Received lux value out of range*/
    SNS_DD_AP3426_ALS_CAL_FAILED_RST = 2, /*ALS CAL error - cannot reset the device*/
    SNS_DD_AP3426_ALS_CAL_FAILED_REG = 3, /*ALS CAL error - cannot write back to registry*/
    SNS_DD_AP3426_ALS_CAL_FAILED_ENB = 4,  /*ALS CAL error - cannot enable (error in set power state)*/
    SNS_DD_AP3426_PROX_CAL_FAILED_CHIP = 5, /*PROX CAL error - cannot read/write the device*/
    SNS_DD_AP3426_PROX_CAL_FAILED_RST  = 6, /*PROX CAL error - cannot reset the device*/
    SNS_DD_AP3426_PROX_CAL_FAILED_REG  = 7, /*PROX CAL error - cannot write back to registry*/
    SNS_DD_AP3426_PROX_CAL_FAILED_ENB  = 8,  /*PROX CAL error - cannot enable (error in set power state)*/
    SNS_DD_AP3426_PROX_CAL_FAILED_DATA = 9  /*PROX CAL error - calibration data is too big*/
} sns_dd_ap3426_test_err_e;

typedef enum
{
  SNS_AP3426_PRX_FAR_AWAY,
  SNS_AP3426_PRX_NEAR_BY,
  SNS_AP3426_PRX_NEAR_BY_UNKNOWN
} sns_dd_ap3426_prx_nearby_type;

typedef  enum
{
  SNS_DD_AP3426_DEV_STOPPED,                         /* 0 */
  SNS_DD_AP3426_DEV_CONFIGURED,                      /* 1 */
  SNS_DD_AP3426_DEV_GET_DATA,                        /* 2 */
  SNS_DD_AP3426_DEV_MAX_STATES                       /* 3 */
}sns_dd_ap3426_device_state_type;

typedef enum
{
  SNS_DD_AP3426_ALSPRX_NV_SOURCE_DEFAULT,
  SNS_DD_AP3426_ALSPRX_NV_SOURCE_REG
}sns_dd_ap3426_nv_source;

/* data structure for light(ALS) driver */
typedef struct
{
  uint32_t                  last_mlux;
  uint16_t                  accuracy;
} sns_dd_ap3426_als_db_type;

/* data structure for NV items */
typedef struct
{
  uint8_t  visible_ratio;   /* visible light transmission ratio of the glass/pipe in % */
  uint8_t  ir_ratio;        /* IR transmission ratio of light glass/pipe in % */
  uint16_t dc_offset;       /* DC offset in ADC count */
  uint16_t thresh_near;     /* near detection threshold in ADC count adjusted by DC offset */
  uint16_t thresh_far;      /* far detection threshold in ADC count adjusted by DC offset */
  uint16_t prx_factor;      /* PRX multiplicative factor in % */
  uint16_t als_factor;      /* ALS multiplicative factor in % */
  /* Device/Driver Specific data fields - start*/
  uint32_t lsc_num;         /* unique number for LSM */
  uint32_t drv_ID;          /* identify specific driver */
  uint32_t calibratePhone;  /* if true, calibrate phone (should only have to done during manufacture) */
  uint32_t device;          /* current sensor */
  uint32_t ALS_thresh_high; /* ALS Interrupt Threshold High in % */
  uint32_t ALS_thresh_low;   /* ALS Interrupt Threshold Low in % */
} sns_dd_ap3426_nv_db_type;

typedef struct
{
  sns_dd_ap3426_nv_db_type         nv_db;                  /* NV item DB type */
  sns_dd_ap3426_nv_source          nv_source;
  sns_ddf_powerstate_e             pwr_mode;
  sns_dd_ap3426_device_state_type  state;
} sns_dd_ap3426_alsprx_common_db_type;

/* State struct for ALS/PRX */
typedef struct {
   sns_ddf_handle_t                     smgr_handle;    /* SDDI handle used to notify_data */
   sns_ddf_handle_t                     port_handle;    /* handle used to access the I2C bus */
   sns_ddf_timer_s                      sns_dd_tmr_obj; /* timer obj */
   uint32_t                             dropped_logs;
   sns_dd_ap3426_als_db_type            sns_dd_als_db;
   sns_dd_ap3426_alsprx_common_db_type  sns_dd_alsprx_common_db;
   bool                                 calibration_mode;    /* calibration mode enable flag */
   uint16_t                             nv_db_size;   /* size of NV item */
   ap3426_chip                          chip;
   sns_ddf_handle_t                     ZBW_port_handle;
   uint32_t                             int_gpio;
   sns_ddf_sensor_data_s*               s_data;
   sns_ddf_sensor_sample_s*             samples;
   bool                                 als_auto_gain_trig;
   sns_ddf_time_t                       als_auto_gain_trig_time;
   uint32_t                             als_auto_gain_delay_time;
} sns_dd_ap3426_alsprx_state_t;

sns_ddf_status_e sns_dd_alsprx_ap3426_init
(
  sns_ddf_handle_t*        dd_handle_ptr,
  sns_ddf_handle_t         smgr_handle,
  sns_ddf_nv_params_s*     nv_params,
  sns_ddf_device_access_s  device_info[],
  uint32_t                 num_devices,
  sns_ddf_memhandler_s*    memhandler,
  sns_ddf_sensor_e*        sensors[],
  uint32_t*                num_sensors
);

sns_ddf_status_e sns_dd_alsprx_ap3426_get_data
(
  sns_ddf_handle_t        dd_handle,
  sns_ddf_sensor_e        sensors[],
  uint32_t                num_sensors,
  sns_ddf_memhandler_s*   memhandler,
  sns_ddf_sensor_data_s*  data[] /* ignored by this async driver */
);

sns_ddf_status_e sns_dd_alsprx_ap3426_set_attrib
(
  sns_ddf_handle_t     dd_handle,
  sns_ddf_sensor_e     sensor,
  sns_ddf_attribute_e  attrib,
  void*                value
);

sns_ddf_status_e sns_dd_alsprx_ap3426_get_attrib
(
  sns_ddf_handle_t     dd_handle,
  sns_ddf_sensor_e     sensor,
  sns_ddf_attribute_e  attrib,
  sns_ddf_memhandler_s* memhandler,
  void**               value,
  uint32_t*            num_elems
);

#if 1
void sns_dd_alsprx_ap3426_handle_timer 
    (
    sns_ddf_handle_t dd_handle, 
    void* arg
    );
#endif

void sns_dd_alsprx_ap3426_handle_irq
(
  sns_ddf_handle_t dd_handle,
  uint32_t          gpio_num,
  sns_ddf_time_t    timestamp
);

sns_ddf_status_e sns_dd_alsprx_ap3426_reset
(
  sns_ddf_handle_t dd_handle
);

sns_ddf_status_e sns_dd_alsprx_ap3426_run_test
(
 sns_ddf_handle_t  dd_handle,
 sns_ddf_sensor_e  sensor,
 sns_ddf_test_e    test,
 uint32_t*         err
);
sns_ddf_status_e sns_dd_alsprx_ap3426_enable_sched_data
(
 sns_ddf_handle_t state,
 sns_ddf_sensor_e sensor,
 bool             enable
 );
 
sns_ddf_status_e sns_dd_alsprx_ap3426_probe(
  sns_ddf_device_access_s* device_info,
  sns_ddf_memhandler_s*    memhandler,
  uint32_t*                num_sensors,
  sns_ddf_sensor_e**       sensors );

sns_ddf_status_e sns_dd_ap3426_als_sensor_samples(sns_dd_ap3426_alsprx_state_t   *state,
                                            sns_ddf_sensor_data_s   *data_ptr,
                                            sns_ddf_memhandler_s    *memhandler,
                                            sns_ddf_sensor_sample_s *sample_data);

sns_ddf_status_e sns_dd_ap3426_prox_sensor_samples(sns_dd_ap3426_alsprx_state_t   *state,
                                             sns_ddf_sensor_data_s   *data_ptr,
                                             sns_ddf_memhandler_s    *memhandler,
                                             sns_ddf_sensor_sample_s *sample_data);

sns_ddf_status_e sns_dd_ap3426_get_mlux(sns_dd_ap3426_alsprx_state_t* state);

sns_ddf_status_e sns_dd_ap3426_get_als_data(sns_dd_ap3426_alsprx_state_t* state);

sns_ddf_status_e sns_dd_ap3426_get_prox_data(sns_dd_ap3426_alsprx_state_t* state);

sns_ddf_status_e sns_dd_alsprx_ap3426_reset
(
  sns_ddf_handle_t dd_handle
);

sns_ddf_status_e sns_dd_ap3426_write_byte(sns_ddf_handle_t port_handle, uint8_t reg, uint8_t* byte_val);

sns_ddf_status_e sns_dd_ap3426_write_word(sns_ddf_handle_t port_handle, uint8_t reg, ap3426_uword_u* word_val);

sns_ddf_status_e sns_dd_ap3426_read_byte(sns_ddf_handle_t port_handle, uint8_t reg, uint8_t* byte_val);

sns_ddf_status_e sns_dd_ap3426_read_word(sns_ddf_handle_t port_handle, uint8_t reg, ap3426_uword_u* word_val);

bool sns_dd_ap3426_als_auto_gain(sns_dd_ap3426_alsprx_state_t* state, uint16_t alsdata);

#endif
