/*==============================================================================

    S E N S O R S   AMBIENT LIGHT AND PROXIMITY  D R I V E R

DESCRIPTION

   Implements the driver for the ap3426 ALS(Light) and Proximity Sensor

Copyright (c) 2015 by QUALCOMM Technologies, Incorporated.  All Rights Reserved.
Qualcomm Technologies Proprietary and Confidential
==============================================================================*/
/*==============================================================================

                      EDIT HISTORY FOR FILE

  This section contains comments describing changes made to the module.
  Notice that changes are listed in reverse chronological order.

$Header: //source/qcom/qct/core/pkg/dsps/rel/2.2/adsp_proc/Sensors/dd/qcom/src/sns_dd_alsprx_ap3426_uimg.c#1 $

***************************************************************************
* REVISON HISTORY
* VERSION   DATE(MM/DD/YY)  WHO     DESCRIPTION
* 3.0       03/10/2017      LO      Refactoring scenario of als auto gain
* 2.0       06/18/2016      LO      Add a compiled flag for enable/disable uImage
* 1.0       12/22/2015      LO      Initial version of driver for AP3426
***************************************************************************

==============================================================================*/
/*============================================================================
                                INCLUDE FILES
============================================================================*/
#include "sns_dd_alsprx_ap3426_priv.h"

#include "sns_memmgr.h"
#include "log_codes.h"
#include "sns_debug_str.h"
#include "sns_debug_api.h"
#include "sns_log_types.h"
#include "sns_log_api.h"


sns_ddf_driver_if_s AP3426_DD_IF =
{
  .init = &sns_dd_alsprx_ap3426_init,
  .get_data = &sns_dd_alsprx_ap3426_get_data,
  .set_attrib = &sns_dd_alsprx_ap3426_set_attrib,
  .get_attrib = &sns_dd_alsprx_ap3426_get_attrib,
  .handle_timer = &sns_dd_alsprx_ap3426_handle_timer,
  .handle_irq = &sns_dd_alsprx_ap3426_handle_irq,
  .reset = &sns_dd_alsprx_ap3426_reset,
  .run_test = &sns_dd_alsprx_ap3426_run_test,  
  .enable_sched_data = &sns_dd_alsprx_ap3426_enable_sched_data,
  .probe = &sns_dd_alsprx_ap3426_probe
};


/*===========================================================================
FUNCTION      sns_dd_ap3426_write_byte

DESCRIPTION   write a byte from device
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_write_byte(sns_ddf_handle_t port_handle, uint8_t reg, uint8_t* byte_val)
{
    sns_ddf_status_e status;
    uint8_t          write_count;   

    status = sns_ddf_write_port(port_handle,
                                reg,
                                byte_val,
                                1,
                                &write_count
                                );
    return status;
}
/*===========================================================================
FUNCTION      sns_dd_ap3426_write_word

DESCRIPTION   write a word from device
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_write_word(sns_ddf_handle_t port_handle, uint8_t reg, ap3426_uword_u* word_val)
{
    sns_ddf_status_e status;
    uint8_t          write_count;

    status = sns_ddf_write_port(port_handle,
                                reg,
                                &word_val->byte[0],
                                2,
                                &write_count
                                );
    return status;
}
/*===========================================================================
FUNCTION      sns_dd_ap3426_read_byte

DESCRIPTION   read a byte from device
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_read_byte(sns_ddf_handle_t port_handle, uint8_t reg, uint8_t* byte_val)
{
    sns_ddf_status_e status;
    uint8_t          read_count;

    status = sns_ddf_read_port(port_handle,
                               reg,
                               byte_val,
                               1,
                               &read_count
                               );
    return status;
}
/*===========================================================================
FUNCTION      sns_dd_ap3426_read_word

DESCRIPTION   read a word from device
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_read_word(sns_ddf_handle_t port_handle, uint8_t reg, ap3426_uword_u* word_val)
{
    sns_ddf_status_e status;
    uint8_t          read_count;
    ap3426_uword_u          word_tmp;

    status = sns_ddf_read_port(port_handle,
                               reg,
                               &word_tmp.byte[0],
                               2,
                               &read_count
                               );
    if(status == SNS_DDF_SUCCESS)
    {
        word_val->word = word_tmp.word;
    }
    return status;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_log

DESCRIPTION   Log the latest sensor data
===========================================================================*/
void sns_dd_alsprx_ap3426_log(
   sns_dd_ap3426_alsprx_state_t* dd_ptr,
   sns_ddf_sensor_e       sensor_type,
   uint32_t               data1,
   uint32_t               data1_q16,
   uint32_t               data2,
   uint32_t               data2_q16,
   uint32_t               raw_data )
{
    sns_err_code_e err_code;
    sns_log_sensor_data_pkt_s *log_struct_ptr;
    log_pkt_t log_pkt_type;
    
#ifndef AP3426_DDK
    log_pkt_type = SNS_LOG_CONVERTED_SENSOR_DATA;
#else
    log_pkt_type = SNS_LOG_CONVERTED_SENSOR_DATA_PUBLIC;
#endif

   DDF_MSG_0(HIGH, "AP3426 Log Data");

   /* Allocate log packet */
   err_code = sns_logpkt_malloc(log_pkt_type,
                                sizeof(sns_log_sensor_data_pkt_s),
                                (void**)&log_struct_ptr);

   if ((err_code == SNS_SUCCESS) && (log_struct_ptr != NULL))
   {
     log_struct_ptr->version = SNS_LOG_SENSOR_DATA_PKT_VERSION;
     log_struct_ptr->sensor_id = sensor_type;
     log_struct_ptr->vendor_id = SNS_DDF_VENDOR_AMS;

     /* Timestamp the log with sample time */
     log_struct_ptr->timestamp = sns_ddf_get_timestamp();

     /* Log the sensor data */
     if (sensor_type == SNS_DDF_SENSOR_AMBIENT)
     {
       log_struct_ptr->num_data_types = 3;
       log_struct_ptr->data[0] = data1;
       log_struct_ptr->data[1] = data1_q16;
       log_struct_ptr->data[2] = raw_data;
      }
     else
     {
       log_struct_ptr->num_data_types = 5;
       log_struct_ptr->data[0] = data1;
       log_struct_ptr->data[1] = data1_q16;
       log_struct_ptr->data[2] = data2;
       log_struct_ptr->data[3] = data2_q16;
       log_struct_ptr->data[4] = raw_data;
     }

     /* Commit log (also frees up the log packet memory) */
     err_code = sns_logpkt_commit(log_pkt_type, log_struct_ptr);
   }
   else
   {
       DDF_MSG_1(HIGH, "AP3426 Log Data - logpkt_malloc failed with err: %d", err_code);
       dd_ptr->dropped_logs++;
   }
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_prx_binary

DESCRIPTION   ensure far or near now
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_prx_binary
(
  sns_dd_ap3426_alsprx_state_t* state
)
{    
    sns_ddf_status_e status = SNS_DDF_SUCCESS;

    if ( state->chip.data.prox.data > state->chip.setup.prox.thresh_high) {
        state->chip.data.prox.event = SNS_AP3426_PRX_NEAR_BY;
    } else if ( state->chip.data.prox.data < state->chip.setup.prox.thresh_low ) {
        state->chip.data.prox.event = SNS_AP3426_PRX_FAR_AWAY;
    }

    return status;
}
/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_set_als_thresh

DESCRIPTION   set als threshold
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_set_als_thresh
(
  sns_dd_ap3426_alsprx_state_t* state
)
{
    uint32_t alsdata, t_low, t_high;
    ap3426_uword_u wordVal;

    sns_ddf_status_e status = SNS_DDF_SUCCESS;

    alsdata = state->chip.data.als.data;
    DDF_MSG_1(MED, "set_als_thresh() alsdata %d",  alsdata );

    if(alsdata < SNS_AP3426_ALS_INT_CNT_L) alsdata = SNS_AP3426_ALS_INT_CNT_L;
    t_low = alsdata - SNS_AP3426_ALS_INT_CNT_L;   /*set to 27 as default*/

    if(alsdata > SNS_AP3426_ALS_INT_CNT_H ) alsdata = SNS_AP3426_ALS_INT_CNT_H;
    t_high = alsdata + SNS_AP3426_ALS_INT_CNT_L;   /*set to 27 as default*/

    wordVal.word = t_low;
    DDF_MSG_1(MED, "set_als_thresh() als t low wordval.word %d",  wordVal.word);
    status = sns_dd_ap3426_write_word(state->port_handle, AP3426_ALS_LTHL, &wordVal);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(HIGH, "set_als_thresh() ALS thresh low set error %d", wordVal.word);
        return status;
    }

    wordVal.word = t_high;
    DDF_MSG_1(MED, "set_als_thresh() als t high wordval.word %d",  wordVal.word);
    status = sns_dd_ap3426_write_word(state->port_handle, AP3426_ALS_HTHL, &wordVal);
    if ( status != SNS_DDF_SUCCESS )
    {
        DDF_MSG_1(HIGH, "set_als_thresh() ALS Thresh set high error %d", wordVal.word);
        return status;
    }

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_get_mlux

DESCRIPTION   set attrib
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_get_mlux(sns_dd_ap3426_alsprx_state_t* state)
{
    uint16_t als_data;
    uint32_t als_lux = 0;    
    uint16_t max_lux = 32767; // depend on user case    

    als_data = state->chip.data.als.data;

    switch (state->chip.setup.als.gain) {
        case AP3426_RANGE_D:
            als_lux = als_data * AP3426_RANGE_D_RATION;
            break;
        case AP3426_RANGE_C:
            als_lux = als_data * AP3426_RANGE_C_RATION;
            break;
        case AP3426_RANGE_B:
            als_lux = als_data * AP3426_RANGE_B_RATION;
            break;
        case AP3426_RANGE_A:
            als_lux = als_data * AP3426_RANGE_A_RATION;
            break;
    }

    state->chip.data.als.lux = als_lux > max_lux?max_lux:als_lux;
    state->chip.data.als.mlux = state->chip.data.als.lux * 1000;
    state->chip.data.als.lux_ok = true;
    DDF_MSG_3(MED, "ap3426_get_mlux() als gain: 0x%02x, als lux: %d, mlux: %d", state->chip.setup.als.gain, state->chip.data.als.lux, state->chip.data.als.mlux);

    return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_get_prox_data

DESCRIPTION   get prox data from device
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_get_prox_data(sns_dd_ap3426_alsprx_state_t* state)
{
    sns_ddf_status_e      status;
    ap3426_uword_u pdata;
    

    status = sns_dd_ap3426_read_word(state->port_handle,
                               AP3426_PX_ADC_LSB,
                               &pdata);
    if(status != SNS_DDF_SUCCESS)
    {
        DDF_MSG_1(HIGH, "get_prox_data() read prx data err: %d", status);
        return status;
    }   

    state->chip.data.prox.data = pdata.word & SNS_AP3426_PRO_DATA_MASK;
    DDF_MSG_2(MED, "get_prox_data() out of get prox data: %d var2: %d", 1158, state->chip.data.prox.data);

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_als_auto_gain

DESCRIPTION   als auto gain
===========================================================================*/
bool sns_dd_ap3426_als_auto_gain(sns_dd_ap3426_alsprx_state_t* state, uint16_t alsdata)
{
    sns_ddf_status_e status = SNS_DDF_SUCCESS;
    uint8_t byteVal;    


    status = sns_dd_ap3426_read_byte(state->port_handle, AP3426_ALS_GAIN, &byteVal);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(HIGH, "als_auto_gain() read als gain err: %d", status);
    }
    state->chip.setup.als.gain = byteVal & SNS_AP3426_ALS_GAIN_MASK;

    if ((alsdata >= AP3426_ALS_AUTO_GAIN_HTHRES) && (state->chip.setup.als.gain > 0x00)) {
        state->als_auto_gain_trig_time = sns_ddf_get_timestamp();
        DDF_MSG_2(MED, "als_auto_gain() als_data %d > auto_auto_gain_Hthres: %d", alsdata, AP3426_ALS_AUTO_GAIN_HTHRES);
        state->chip.setup.als.gain = state->chip.setup.als.gain - 0x10;
        DDF_MSG_1(HIGH, "als_auto_gain() next gain: 0x%02x", state->chip.setup.als.gain);
        byteVal = state->chip.setup.als.gain;
        status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_ALS_GAIN, &byteVal);
        if (status != SNS_DDF_SUCCESS) {
            DDF_MSG_1(MED, "als_auto_gain() write ALS gain err: %d", status);
        }       
    } else if ((alsdata <= AP3426_ALS_AUTO_GAIN_LTHRES) && (state->chip.setup.als.gain < 0x30)) {
        state->als_auto_gain_trig_time = sns_ddf_get_timestamp();
        DDF_MSG_2(MED, "als_auto_gain() als_data %d < auto_gain_Lthres: %d", alsdata, AP3426_ALS_AUTO_GAIN_LTHRES);
        state->chip.setup.als.gain = state->chip.setup.als.gain + 0x10;
        DDF_MSG_1(HIGH, "als_auto_gain() next gain: 0x%02x", state->chip.setup.als.gain);
        byteVal = state->chip.setup.als.gain;
        status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_ALS_GAIN, &byteVal);
        if (status != SNS_DDF_SUCCESS) {
            DDF_MSG_1(MED, "als_auto_gain() write ALS gain err: %d", status);
        }       
    } else {
        state->als_auto_gain_trig_time = 0;     
        return false;
    }
    
    DDF_MSG_1(MED, "als_auto_gain() auto gain triggered set: 0x%02x", state->chip.setup.als.gain);

    return true;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_get_als_data

DESCRIPTION   get als data from device for ap3426 only
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_get_als_data(sns_dd_ap3426_alsprx_state_t* state)
{
    sns_ddf_status_e      status;
    ap3426_uword_u alsdata;


    status = sns_dd_ap3426_read_word(state->port_handle,
                            AP3426_ALS_LDATA_L,
                               &alsdata);
    if(status != SNS_DDF_SUCCESS)
    {
        DDF_MSG_1(HIGH, "get_als_data() read als data err: %d", status);
        return status;
    }

#if AP3426_ALS_AUTO_GAIN_EN
    state->als_auto_gain_trig = sns_dd_ap3426_als_auto_gain(state, alsdata.word);
    if (!state->als_auto_gain_trig) {
        state->chip.data.als.data = alsdata.word;  
    }
#else    
    state->chip.data.als.data = alsdata.word;
#endif

    DDF_MSG_3(MED, "get_als_data() alsdata: %d, als gain: 0x%02x, auto_gain_trig: %d", state->chip.data.als.data, state->chip.setup.als.gain, state->als_auto_gain_trig);

    return status;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_prox_sensor_samples

DESCRIPTION   prox report handle
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_prox_sensor_samples(sns_dd_ap3426_alsprx_state_t   *state,
                                             sns_ddf_sensor_data_s   *data_ptr,
                                             sns_ddf_memhandler_s    *memhandler,
                                             sns_ddf_sensor_sample_s *sample_data)
{
    uint32_t num_samples = 2;

    DDF_MSG_1(MED, "prox_sensor_samples() AP3426 var1: %d", 1161);

    if(memhandler != NULL)
    {
        data_ptr->samples = sns_ddf_memhandler_malloc_ex(memhandler,
                                                      num_samples * sizeof(sns_ddf_sensor_sample_s), state->smgr_handle);
        if(data_ptr->samples == NULL)
        {
            DDF_MSG_1(MED, "prox_sensor_samples() AP3426 var1: %d", 1162);
            return SNS_DDF_ENOMEM;
        }
    }
    else if(sample_data != NULL)
    {
        data_ptr->samples = sample_data;
    }
    else
    {
        DDF_MSG_1(MED, "prox_sensor_samples() AP3426 var1: %d", 1163);
        return SNS_DDF_EINVALID_PARAM;
    }

    data_ptr->samples[0].sample = FX_CONV_Q16(state->chip.data.prox.event, 0);
    data_ptr->samples[0].status = SNS_DDF_SUCCESS;
    data_ptr->samples[1].sample = state->chip.data.prox.data;
    data_ptr->samples[1].status = SNS_DDF_SUCCESS;
    data_ptr->num_samples       = num_samples;

    DDF_MSG_3(FATAL, "prox_sensor_samples() TMD PRX Data: Event:%d  Reported Data: %d  Raw Data: %d", state->chip.data.prox.event, data_ptr->samples[0].sample, state->chip.data.prox.data);
    sns_dd_alsprx_ap3426_log(state, SNS_DDF_SENSOR_PROXIMITY, 
                        state->chip.data.prox.event, data_ptr->samples[0].sample, 0, 0, (uint32_t) state->chip.data.prox.data);

    return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_als_sensor_samples

DESCRIPTION   als report handle
===========================================================================*/
sns_ddf_status_e sns_dd_ap3426_als_sensor_samples(sns_dd_ap3426_alsprx_state_t    *state,
                                            sns_ddf_sensor_data_s    *data_ptr,
                                            sns_ddf_memhandler_s     *memhandler,
                                             sns_ddf_sensor_sample_s *sample_data)
{
    uint32_t num_samples = 2;

    if(memhandler != NULL)
    {
        data_ptr->samples = sns_ddf_memhandler_malloc_ex(
                                                      memhandler,
                                                      num_samples * sizeof(sns_ddf_sensor_sample_s), state->smgr_handle);
        if(data_ptr->samples == NULL)
        {
            DDF_MSG_1(MED, "als_sensor_samples() AP3426 var1: %d", 1165);
            return SNS_DDF_ENOMEM;
        }
    }
    else if(sample_data != NULL)
    {
        data_ptr->samples = sample_data;
    }
    else
    {
        DDF_MSG_1(MED, "als_sensor_samples() AP3426 var1: %d", 1166);
        return SNS_DDF_EINVALID_PARAM;
    }

    data_ptr->samples[0].sample = FX_CONV_Q16(state->chip.data.als.lux, 0);
    data_ptr->samples[0].status = SNS_DDF_SUCCESS;
    data_ptr->samples[1].sample = state->chip.data.als.data;
    data_ptr->samples[1].status = SNS_DDF_SUCCESS;
    data_ptr->num_samples       = num_samples;

    sns_dd_alsprx_ap3426_log(state, SNS_DDF_SENSOR_AMBIENT,
                              state->chip.data.als.lux, data_ptr->samples[0].sample, 0, 0, (uint32_t) state->chip.data.als.data);

    return SNS_DDF_SUCCESS;
}

/*===========================================================================
FUNCTION      sns_dd_ap3426_read_status_reg

DESCRIPTION   get data and report
===========================================================================*/
uint8_t sns_dd_ap3426_read_status_reg(sns_dd_ap3426_alsprx_state_t* state, sns_ddf_time_t timestamp, AP3426_SIG_TYPE sig_type)
{
    uint8_t  statusReg;
    uint8_t  numOfSensors;
    uint8_t  i;
    uint8_t  obj;
    sns_ddf_status_e status;
    sns_ddf_sensor_e sensors[2];
    numOfSensors  = 0;

    status = sns_dd_ap3426_read_byte(state->port_handle,
                      AP3426_INT_STATUS,
                      &statusReg);
    if (status != SNS_DDF_SUCCESS) {
        DDF_MSG_1(MED, "AP3426 in read_status_reg  read int_status register err: %d", status);
        statusReg = AP3426_STA_PRX_INTR | AP3426_STA_ALS_INTR;
        return statusReg;
    }

    DDF_MSG_3(MED, "ap3426_read_status_reg()  status_reg is: 0x%02x, prx_en: %d, als_en: %d" , statusReg, state->chip.irq_prx_en, state->chip.irq_als_en);

   if ((statusReg & AP3426_STA_PRX_INTR) && state->chip.irq_prx_en)
   {
        /* PROX interrupt ensure obj near or far */
        obj = (statusReg & SNS_AP3426_PRO_OBJ_MASK) >> SNS_AP3426_PRO_OBJ_SHIFT;
        if (obj == SNS_AP3426_PRX_NEAR_BY) {
            state->chip.data.prox.event = SNS_AP3426_PRX_NEAR_BY;
        } else {
            state->chip.data.prox.event = SNS_AP3426_PRX_FAR_AWAY;
        }

        sns_dd_ap3426_get_prox_data(state);

        sensors[numOfSensors] = SNS_DDF_SENSOR_PROXIMITY;
        numOfSensors++;
    }

    if ((statusReg & AP3426_STA_ALS_INTR) && state->chip.irq_als_en)
    {
#if AP3426_ALS_AUTO_GAIN_EN
        if (state->als_auto_gain_trig) {
            if ((timestamp - state->als_auto_gain_trig_time) <= state->als_auto_gain_delay_time) {
                DDF_MSG_2(HIGH, "ap3426_read_status_reg() als auto gain waiting time triggered, pass time: %d, als_auto_gain_usdelay: %d", 
                                 (timestamp - state->als_auto_gain_trig_time), state->als_auto_gain_delay_time);                 
                goto exit_get_als_data;                 
            }
        }
#endif
    
        /* Read ALS data */
        sns_dd_ap3426_get_als_data(state);
#if AP3426_ALS_AUTO_GAIN_EN
        if (state->als_auto_gain_trig) {
            DDF_MSG_0(HIGH, "ap3426_read_status_reg() als auto gain triggered");
            goto exit_get_als_data;
        }
#endif
        sns_dd_alsprx_ap3426_set_als_thresh(state); /* set als threshold to be +/- 20% of current value for next interrupt. */     
        /* compute LUX */
        sns_dd_ap3426_get_mlux(state);
        sensors[numOfSensors] = SNS_DDF_SENSOR_AMBIENT;
        numOfSensors++;

        DDF_MSG_2(MED, "ap3426_read_status_reg() ALS int numOfSensors: %d lux: %d", numOfSensors, state->chip.data.als.lux);            
    }

exit_get_als_data:
    if (numOfSensors > 0)
    {
        /* Had at least one interrupt */
        memset(state->s_data, 0, sizeof(*state->s_data));
        for (i = 0; i < numOfSensors; i++)
        {           
            state->s_data[i].sensor    = sensors[i];
            state->s_data[i].status    = SNS_DDF_SUCCESS;
            state->s_data[i].timestamp = timestamp;
            memset(state->samples, 0, sizeof(*state->samples));
            switch(sensors[i])
            {
            case SNS_DDF_SENSOR_PROXIMITY:
                sns_dd_ap3426_prox_sensor_samples(state, &state->s_data[i], NULL, state->samples);
                break;

            case SNS_DDF_SENSOR_AMBIENT:
                sns_dd_ap3426_als_sensor_samples(state, &state->s_data[i], NULL, state->samples);
                break;

            default:
                DDF_MSG_2(MED, "ap3426_read_status_reg() No sensor available: %d, return statusReg: 0x%02x", 1119, statusReg);
                return statusReg;
            }
            sns_ddf_smgr_notify_data(state->smgr_handle, &state->s_data[i], 1);
        }
    }

    DDF_MSG_1(MED, "exit ap3426_read_status_reg(), return statusReg: 0x%02x", statusReg);

    return statusReg;
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_handle_irq

DESCRIPTION   irq handle
===========================================================================*/
void sns_dd_alsprx_ap3426_handle_irq
(
  sns_ddf_handle_t  handle,
  uint32_t          gpio_num,
  sns_ddf_time_t    timestamp
)
{
    sns_dd_ap3426_alsprx_state_t* state = (sns_dd_ap3426_alsprx_state_t*)handle;
    uint8_t int_status, i = 0;
    sns_ddf_status_e status = SNS_DDF_SUCCESS;  

    
    DDF_MSG_1(MED, "AP3426 in handle irq: %d, start", 1124);    
    
    int_status = sns_dd_ap3426_read_status_reg(state, timestamp, AP3426_IRQ_TYPE);

    /* Clear Interrupt Flag */
    if ((int_status & AP3426_STA_PRX_INTR)) {
        int_status &= ~AP3426_STA_PRX_INTR;
    }
    if ((int_status & AP3426_STA_ALS_INTR)) {
        int_status &= ~AP3426_STA_ALS_INTR;
    }
    DDF_MSG_1(MED, "AP3426 in handle irq: write value: 0x%02x to int_status register", int_status);
    while ((status = sns_dd_ap3426_write_byte(state->port_handle, AP3426_INT_STATUS, &int_status)) != SNS_DDF_SUCCESS) {
        if (i == 10) {
            DDF_MSG_1(MED, "ap3426_handle_irq() write int_status register err: %d", status);
            break;
        }
        i++;
        sns_ddf_delay(5000);
    }
    /* Clear Interrupt Flag */  
}

void sns_dd_alsprx_ap3426_handle_timer
(
    sns_ddf_handle_t handle,
    void* arg
)
{
    //sns_dd_ap3426_alsprx_state_t* state = (sns_dd_ap3426_alsprx_state_t*)handle;  
}

/*===========================================================================
FUNCTION      sns_dd_alsprx_ap3426_get_data

DESCRIPTION   get als and prox data
===========================================================================*/
sns_ddf_status_e sns_dd_alsprx_ap3426_get_data
(
  sns_ddf_handle_t        dd_handle,
  sns_ddf_sensor_e        sensors[],
  uint32_t                num_sensors,
  sns_ddf_memhandler_s*   memhandler,
  sns_ddf_sensor_data_s*  data[] /* ignored by this async driver */
)
{
    uint8_t i;
    sns_ddf_time_t        timestamp;
    sns_ddf_status_e      status;
    sns_ddf_sensor_data_s *data_ptr;
    sns_dd_ap3426_alsprx_state_t *state = dd_handle;


    timestamp = sns_ddf_get_timestamp();
    data_ptr  = sns_ddf_memhandler_malloc_ex(
                                         memhandler,
                                         num_sensors * sizeof(sns_ddf_sensor_data_s), state->smgr_handle);
    if (data_ptr == NULL)
    {
        DDF_MSG_1(HIGH, "ap3426_get_data() Get Data malloc error, Size: %d", (num_sensors * sizeof(sns_ddf_sensor_data_s)));
        return SNS_DDF_ENOMEM;
    }
    *data = data_ptr;   

    for (i = 0; i < num_sensors; i++)
    {
        data_ptr[i].sensor    = sensors[i];
        data_ptr[i].status    = SNS_DDF_SUCCESS;
        data_ptr[i].timestamp = timestamp;

        switch(sensors[i])
        {
        case SNS_DDF_SENSOR_PROXIMITY:            
            DDF_MSG_0(HIGH, "ap3426_get_data() Get prx Data");
            if ((status = sns_dd_ap3426_get_prox_data(state)) != SNS_DDF_SUCCESS) 
            {
                DDF_MSG_1(HIGH, "ap3426_get_data() Get prox raw Data err: %d", status);
                return status;
            }

            if ((status = sns_dd_alsprx_ap3426_prx_binary(state)) != SNS_DDF_SUCCESS)
            {
                DDF_MSG_1(HIGH, "ap3426_get_data() Get prox event err: %d", status);
                return status;
            }
            DDF_MSG_0(HIGH, "ap3426_get_data(), returning prx data to smgr");
            sns_dd_ap3426_prox_sensor_samples(state, &data_ptr[i], memhandler, NULL);
            break;

        case SNS_DDF_SENSOR_AMBIENT:
            DDF_MSG_0(HIGH, "ap3426_get_data() Get als Data");

#if AP3426_ALS_AUTO_GAIN_EN
            if (state->als_auto_gain_trig) {
                if ((timestamp - state->als_auto_gain_trig_time) <= state->als_auto_gain_delay_time) {
                    DDF_MSG_2(HIGH, "ap3426_get_data() als auto gain waiting time triggered return, pass time: %d, als_auto_gain_usdelay: %d", 
                                     (timestamp - state->als_auto_gain_trig_time), state->als_auto_gain_delay_time);                 
                    break;                  
                }
            }
#endif

            if ((status = sns_dd_ap3426_get_als_data(state)) != SNS_DDF_SUCCESS)
            {
                DDF_MSG_1(HIGH, "ap3426_get_data() Get als Data err: %d", status);
                return status;
            }

#if AP3426_ALS_AUTO_GAIN_EN
            if (state->als_auto_gain_trig) {
                DDF_MSG_0(HIGH, "ap3426_get_data() als auto gain triggered return directly");               
                break;
            }
#endif

            if ((status = sns_dd_ap3426_get_mlux(state)) != SNS_DDF_SUCCESS)
            {
                DDF_MSG_1(HIGH, "ap3426_get_data() Get als lux err: %d", status);
                return status;
            }
            DDF_MSG_0(HIGH, "ap3426_get_data() returning als data to smgr");
            sns_dd_ap3426_als_sensor_samples(state, &data_ptr[i], memhandler, NULL);
            break;

        default:
            DDF_MSG_0(MED, "ap3426_get_data() Neither ALS or Prox called");
         break;
        }
    }

    return SNS_DDF_SUCCESS;
}

